package com.uas.platform.b2b.dao;

import com.uas.platform.b2b.model.NotExistProduct;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Repository;

import javax.transaction.Transactional;
import java.util.List;

/**
 * 平台不存在的ERP物料编号数据库操作
 *
 * Created by hejq on 2018-03-14.
 */
@Repository
public interface NotExistProductDao extends JpaSpecificationExecutor<NotExistProduct>, JpaRepository<NotExistProduct, Long> {

    /**
     * 通过企业UU和下载状态查询平台不存在的物料信息
     *
     * @param enuu 企业UU
     * @param status 下载状态
     * @return
     */
    List<NotExistProduct> findByEnuuAndStatus(Long enuu, Integer status);

    /**
     * 更新下载状态
     *
     * @param id id
     */
    @Modifying
    @Transactional
    @Query("update NotExistProduct set status = 203 where id = :id")
    void updateStatus(@Param("id") Long id);

    /**
     * 通过企业和物料编号查询是否存在该条记录
     *
     * @param enuu 企业UU
     * @param code 物料编号
     * @return
     */
    List<NotExistProduct> findByEnuuAndCode(Long enuu, String code);
}
