package com.uas.platform.b2b.dao;

import com.uas.platform.b2b.model.PurchaseAccept;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Repository;

import java.util.List;

@Repository
public interface PurchaseAcceptDao extends JpaSpecificationExecutor<PurchaseAccept>, JpaRepository<PurchaseAccept, Long> {

	/**
	 * 按所属企业及编号查找采购验收单
	 * 
	 * @param enUU
	 * @param code
	 * @return
	 */
	public List<PurchaseAccept> findByEnUUAndCode(Long enUU, String code);
	
	/**
	 * 卖家根据卖家UU号和下载状态获取需要下载的客户采购验收入库单
	 * @return
	 */
	public List<PurchaseAccept> findByVendUUAndSendStatus(Long vendUU, Short sendStatus);
	
	/**
	 * 按卖家UU、反过账状态查找
	 * @param vendUU
	 * @param sendStatus
	 * @param nonPosting
	 * @return
	 */
	public List<PurchaseAccept> findByVendUUAndNonPosting(Long vendUU, Short nonPosting);

	/**
	 * 根据供应商UU号获取客户采购验退单数量
	 * @param vendUU
	 * @return
	 */
	@Query("select count(p) from PurchaseAccept p where p.vendUU = :vendUU")
	public long countByVendUU(@Param("vendUU")Long vendUU);

	/**
	 * 通过enUU和erpId查询对应的单据
	 *
	 * @param enUU 企业UU
	 * @param sourceId 对应的erpId
	 * @return
	 */
	List<PurchaseAccept> findByEnUUAndErpId(Long enUU, Long sourceId);
}
