package com.uas.platform.b2b.erp.controller;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import com.uas.platform.b2b.erp.model.OutSourceProdInOutDown;
import com.uas.platform.b2b.erp.service.OutSourceProdInOutDownService;
import com.uas.platform.b2b.erp.support.ErpBufferedLogger;
import com.uas.platform.b2b.model.MakeAccept;
import com.uas.platform.b2b.service.MakeAcceptService;
import com.uas.platform.core.logging.BufferedLoggerManager;

/**
 * 对卖家ERP的数据接口<br>
 * 
 * @author aof
 * @date 2015年8月28日
 */
@Controller
@RequestMapping("/erp/outSource/prodInOut")
public class OutSourceProdInOutDownController {

	@Autowired
	private MakeAcceptService makeAcceptService;

	@Autowired
	private OutSourceProdInOutDownService outSourceProdInOutDownService;
	
	private final static ErpBufferedLogger logger = BufferedLoggerManager.getLogger(ErpBufferedLogger.class);

	/**
	 * 卖家ERP从平台获取未下载的客户委外验收单
	 * 
	 * @return
	 */
	@RequestMapping(method = RequestMethod.GET)
	@ResponseBody
	public List<OutSourceProdInOutDown> getOutSourceProdInOuts() {
		List<OutSourceProdInOutDown> downs = outSourceProdInOutDownService.convertOutSourceAccepts(makeAcceptService.findNotUpload());
		logger.log("客户委外验收单", "下载已过账客户委外验收单", downs.size());
		return downs;
	}

	/**
	 * 平台的客户委外验收单传到供应商ERP之后，修改平台里面的客户委外验收单的上传状态
	 * 
	 * @throws UnsupportedEncodingException
	 */
	@RequestMapping(method = RequestMethod.POST)
	@ResponseBody
	public void onSourceDownChangeSuccess(@RequestParam("data") String data) throws UnsupportedEncodingException {
		makeAcceptService.onUploadSuccess(URLDecoder.decode(data, "UTF-8").split(","));
	}

	/**
	 * 卖家ERP从平台获取未下载客户已反过帐的的客户委外验收单
	 * 
	 * @return
	 */
	@RequestMapping(value = "/nonPosting", method = RequestMethod.GET)
	@ResponseBody
	public List<Long> getNonPosting() {
		List<Long> ids = new ArrayList<Long>();
		List<MakeAccept> accepts = makeAcceptService.getNonPosting();
		if (!CollectionUtils.isEmpty(accepts)) {
			for (MakeAccept accept : accepts) {
				ids.add(accept.getId());
			}
		}
		logger.log("客户委外验收单", "下载反过帐的客户委外验收单", ids.size());
		return ids;
	}

	/**
	 * 平台的客户委外验收单传到供应商ERP之后，修改平台里面的客户委外验收单的上传状态
	 * 
	 * @return
	 * @throws UnsupportedEncodingException
	 */
	@RequestMapping(value = "/nonPosting", method = RequestMethod.POST)
	@ResponseBody
	public void onNonPostingDownSuccess(@RequestParam("data") String data) throws UnsupportedEncodingException {
		makeAcceptService.onNonPostingUploadSuccess(URLDecoder.decode(data, "UTF-8").split(","));
	}

}
