package com.uas.platform.b2b.erp.controller;

import com.uas.platform.b2b.erp.model.PurchaseProdInOut;
import com.uas.platform.b2b.erp.service.PurchaseProdInOutService;
import com.uas.platform.b2b.erp.support.ErpBufferedLogger;
import com.uas.platform.b2b.service.PurchaseBadInService;
import com.uas.platform.core.logging.BufferedLoggerManager;
import com.uas.platform.core.util.serializer.FlexJsonUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.List;

/**
 * 对买家ERP的数据接口<br>
 * 买家不良品入库单接口
 * 
 * @author suntg
 * 
 */
@Controller
@RequestMapping("/erp/purchase/prodBadIn")
public class PurchaseProdBadInController {

	@Autowired
	PurchaseBadInService purchaseBadInService;
	
	@Autowired
	PurchaseProdInOutService purchaseProdInOutService;

	private final static ErpBufferedLogger logger = BufferedLoggerManager.getLogger(ErpBufferedLogger.class);

	/**
	 * 将ERP的不良品入库单写到平台
	 * 
	 * @param data
	 * @return
	 * @throws UnsupportedEncodingException
	 */
	@RequestMapping(method = RequestMethod.POST)
	@ResponseBody
	public List<PurchaseProdInOut> savePurchases(@RequestParam("data") String data) throws UnsupportedEncodingException {
		String jsonStr = URLDecoder.decode(data, "UTF-8");
		List<PurchaseProdInOut> prodInOuts = FlexJsonUtils.fromJsonArray(jsonStr, PurchaseProdInOut.class);
		purchaseBadInService.save(purchaseProdInOutService.convertBadIns(prodInOuts));
		logger.log("不良品入库单", "上传不良品入库单", prodInOuts.size());
		return purchaseProdInOutService.getProdBadInB2bId(prodInOuts);
	}

	/**
	 * 将ERP的反过账的不良品入库单写到平台
	 * 
	 * @param data
	 * @return
	 * @throws UnsupportedEncodingException
	 */
	@RequestMapping(value="/nonPosting", method = RequestMethod.POST)
	@ResponseBody
	public void nonPostingProdInOuts(@RequestParam("data") String data) throws UnsupportedEncodingException {
		String jsonStr = URLDecoder.decode(data, "UTF-8");
		List<PurchaseProdInOut> prodInOuts = FlexJsonUtils.fromJsonArray(jsonStr, PurchaseProdInOut.class);
		purchaseBadInService.nonPosting(purchaseProdInOutService.convertNonPostingBadIns(prodInOuts));
		logger.log("反过账的不良品入库单", "上传反过账的不良品入库单", prodInOuts.size());
	}

}
