package com.uas.platform.b2b.erp.controller;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import com.uas.platform.b2b.erp.model.SaleProdInOutDown;
import com.uas.platform.b2b.erp.service.SaleProdInOutDownService;
import com.uas.platform.b2b.erp.support.ErpBufferedLogger;
import com.uas.platform.b2b.model.PurchaseBadIn;
import com.uas.platform.b2b.service.PurchaseBadInService;
import com.uas.platform.core.logging.BufferedLoggerManager;

/**
 * 对卖家ERP的数据接口<br>
 * 客户不良品入库单
 * @author suntg
 *
 */
@Controller
@RequestMapping("/erp/sale/prodBadIn")
public class SaleProdBadInDownController {

	@Autowired
	private PurchaseBadInService purchaseBadInService;

	@Autowired
	private SaleProdInOutDownService saleProdInOutDownService;

	private final static ErpBufferedLogger logger = BufferedLoggerManager.getLogger(ErpBufferedLogger.class);

	/**
	 * 卖家ERP从平台获取未下载的客户不良品入库单
	 * 
	 * @return
	 */
	@RequestMapping(method = RequestMethod.GET)
	@ResponseBody
	public List<SaleProdInOutDown> getOrderChanges() {
		List<SaleProdInOutDown> downs = saleProdInOutDownService.convertPurchaseBadIns(purchaseBadInService.findNotUpload());
		logger.log("客户不良品入库单", "获取未下载的客户不良品入库单", downs.size());
		return downs;
	}

	/**
	 * 平台的客户不良品入库单传到供应商ERP之后，修改平台里面的客户不良品入库单的上传状态
	 * 
	 * @return
	 * @throws UnsupportedEncodingException
	 */
	@RequestMapping(method = RequestMethod.POST)
	@ResponseBody
	public void onSaleDownChangeSuccess(@RequestParam("data") String data) throws UnsupportedEncodingException {
		purchaseBadInService.onUploadSuccess(URLDecoder.decode(data, "UTF-8").split(","));
	}

	/**
	 * 卖家ERP从平台获取未下载客户已反过帐的的客户不良品入库单
	 * 
	 * @return
	 */
	@RequestMapping(value="/nonPosting", method = RequestMethod.GET)
	@ResponseBody
	public List<Long> getNonPosting() {
		List<Long> ids = new ArrayList<Long>();
		List<PurchaseBadIn> badIns = purchaseBadInService.getNonPosting();
		if(! CollectionUtils.isEmpty(badIns)) {
			for(PurchaseBadIn badIn : badIns) {
				ids.add(badIn.getId());
			}
		}
		logger.log("反过帐的客户不良品入库单", "获取未下载的客户客户不良品入库单", badIns.size());
		return ids;
	}

	/**
	 * 平台的客户采购验收单传到供应商ERP之后，修改平台里面的客户不良品入库单的上传状态
	 * 
	 * @return
	 * @throws UnsupportedEncodingException
	 */
	@RequestMapping(value="/nonPosting", method = RequestMethod.POST)
	@ResponseBody
	public void onNonPostingDownSuccess(@RequestParam("data") String data) throws UnsupportedEncodingException {
		purchaseBadInService.onNonPostingUploadSuccess(URLDecoder.decode(data, "UTF-8").split(","));
	}

}