package com.uas.platform.b2b.erp.controller;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import com.uas.platform.b2b.erp.model.SaleProdInOutDown;
import com.uas.platform.b2b.erp.service.SaleProdInOutDownService;
import com.uas.platform.b2b.erp.support.ErpBufferedLogger;
import com.uas.platform.b2b.model.PurchaseReturn;
import com.uas.platform.b2b.service.PurchaseReturnService;
import com.uas.platform.core.logging.BufferedLoggerManager;

/**
 * 对卖家ERP的数据接口<br>
 * 
 * @author suntg
 *
 */
@Controller
@RequestMapping("/erp/sale/prodReturn")
public class SaleProdReturnDownController {

	@Autowired
	private PurchaseReturnService purchaseReturnService;

	@Autowired
	private SaleProdInOutDownService saleProdInOutDownService;

	private final static ErpBufferedLogger logger = BufferedLoggerManager.getLogger(ErpBufferedLogger.class);

	/**
	 * 卖家ERP从平台获取未下载的客户采购验退单
	 * 
	 * @return
	 */
	@RequestMapping(method = RequestMethod.GET)
	@ResponseBody
	public List<SaleProdInOutDown> getOrderChanges() {
		List<SaleProdInOutDown> prodInOuts = saleProdInOutDownService.convertPurchaseReturns(purchaseReturnService.findNotUpload());
		logger.log("客户采购验退单", "下载客户采购验退单", prodInOuts.size());
		return prodInOuts;
	}

	/**
	 * 平台的客户采购验退单传到供应商ERP之后，修改平台里面的客户采购验退单的上传状态
	 * 
	 * @return
	 * @throws UnsupportedEncodingException
	 */
	@RequestMapping(method = RequestMethod.POST)
	@ResponseBody
	public void onSaleDownChangeSuccess(@RequestParam("data") String data) throws UnsupportedEncodingException {
		purchaseReturnService.onUploadSuccess(URLDecoder.decode(data, "UTF-8").split(","));
	}

	/**
	 * 卖家ERP从平台获取未下载客户已反过帐的的客户采购验退单
	 * 
	 * @return
	 */
	@RequestMapping(value="/nonPosting", method = RequestMethod.GET)
	@ResponseBody
	public List<Long> getNonPosting() {
		List<Long> ids = new ArrayList<Long>();
		List<PurchaseReturn> purchaseReturns = purchaseReturnService.getNonPosting();
		if(! CollectionUtils.isEmpty(purchaseReturns)) {
			for(PurchaseReturn purchaseReturn : purchaseReturns) {
				ids.add(purchaseReturn.getId());
			}
		}
		logger.log("反过帐的客户采购验退单", "下载反过帐的客户采购验退单", purchaseReturns.size());
		return ids;
	}

	/**
	 * 平台的客户采购验收单传到供应商ERP之后，修改平台里面的客户采购验退单的上传状态
	 * 
	 * @return
	 * @throws UnsupportedEncodingException
	 */
	@RequestMapping(value="/nonPosting", method = RequestMethod.POST)
	@ResponseBody
	public void onNonPostingDownSuccess(@RequestParam("data") String data) throws UnsupportedEncodingException {
		purchaseReturnService.onNonPostingUploadSuccess(URLDecoder.decode(data, "UTF-8").split(","));
	}

}