package com.uas.platform.b2b.erp.controller;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import com.uas.platform.b2b.erp.model.SaleQuaMRB;
import com.uas.platform.b2b.erp.service.SaleQuaMRBService;
import com.uas.platform.b2b.erp.support.ErpBufferedLogger;
import com.uas.platform.b2b.service.PurchaseMRBService;
import com.uas.platform.core.logging.BufferedLoggerManager;

/**
 * 对卖家ERP的数据接口<br>
 * 客户MRB单
 * @author suntg
 *
 */
@Controller
@RequestMapping("/erp/sale/MRB")
public class SaleQuaMRBController {

	@Autowired
	private PurchaseMRBService purchaseMRBService;

	@Autowired
	private SaleQuaMRBService saleQuaMRBService;

	private final static ErpBufferedLogger logger = BufferedLoggerManager.getLogger(ErpBufferedLogger.class);

	/**
	 * 卖家ERP从平台获取未下载的客户不良品入库单
	 * 
	 * @return
	 */
	@RequestMapping(method = RequestMethod.GET)
	@ResponseBody
	public List<SaleQuaMRB> getSaleQuaMRBs() {
		List<SaleQuaMRB> quaMRB = saleQuaMRBService.convertPurchaseMRBs(purchaseMRBService.findNotUpload());
		logger.log("客户不良品入库单", "下载客户不良品入库单", quaMRB.size());
		return quaMRB;
	}

	/**
	 * 平台的客户不良品入库单传到供应商ERP之后，修改平台里面的客户不良品入库单的上传状态
	 * 
	 * @return
	 * @throws UnsupportedEncodingException
	 */
	@RequestMapping(method = RequestMethod.POST)
	@ResponseBody
	public void onSaleQuaMRBDownSuccess(@RequestParam("data") String data) throws UnsupportedEncodingException {
		purchaseMRBService.onUploadSuccess(URLDecoder.decode(data, "UTF-8").split(","));
	}

}