package com.uas.platform.b2b.erp.service;

import com.uas.platform.b2b.erp.model.Inquiry;
import com.uas.platform.b2b.erp.model.InquiryDecide;
import com.uas.platform.b2b.erp.model.InquiryDetail;
import com.uas.platform.b2b.model.PurchaseInquiryItem;
import com.uas.platform.b2b.model.PurchaseInquiryItemTemp;
import com.uas.platform.b2b.model.SaleQuotation;
import com.uas.platform.b2b.model.SaleQuotationItem;

import java.util.List;

public interface InquiryService {

	/**
	 * 将ERP系统的采购询价，转为平台的询价单
	 * 
	 * @param inquiries
	 * @return
	 */
	List<PurchaseInquiryItemTemp> convertInquiry(List<Inquiry> inquiries);

	/**
	 * 将平台的报价信息，转为买家ERP的报价信息
	 * 
	 * @param inquiryItems
	 * @return
	 */
	List<InquiryDetail> convertPurchaseInquiryReply(List<PurchaseInquiryItem> inquiryItems);

	/**
	 * 将买家ERP对报价信息的采纳结果，转到平台的询价明细里面
	 * 
	 * @param decides
	 * @return
	 */
	List<PurchaseInquiryItem> convertInquiryDecide(List<InquiryDecide> decides);

	/**
	 * 将平台的主动报价，转为买家的主动报价
	 * 
	 * @param quotations
	 * @return
	 */
	List<Inquiry> convertSaleQuotation(List<SaleQuotation> quotations);

	/**
	 * 将买家ERP对主动报价的采纳结果，转到平台的主动报价明细里面
	 * 
	 * @param decides
	 * @return
	 */
	List<SaleQuotationItem> convertQuotationDecide(List<InquiryDecide> decides);

    /**
     * 保存自动询价单明细
     *
     * @param details
     */
    void save(List<InquiryDetail> details);

	/**
	 * 通过ERP上传的数据查询平台存储的id
	 *
	 * @param inquiryList 询价信息
	 * @return 获取结果的询价信息
	 */
	List<Inquiry> getB2bId(List<Inquiry> inquiryList);

}
