package com.uas.platform.b2b.mobile.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;

import com.uas.platform.b2b.mobile.model.Notice;
import com.uas.platform.b2b.mobile.service.NoticeService;
import com.uas.platform.b2b.service.PurchaseNoticeService;
import com.uas.platform.b2b.support.SystemSession;
import com.uas.platform.core.model.PageInfo;
import com.uas.platform.core.model.PageParams;
import com.uas.platform.core.persistence.criteria.PredicateUtils;

/**
 * 对移动端的数据接口<br>
 * 卖家对收到的客户发货提醒的处理
 * 
 * @author yingp
 * 
 */
@Controller("mobile.NoticeController")
@RequestMapping("/mobile/sale/notice")
public class NoticeController {

	@Autowired
	private NoticeService noticeService;

	@Autowired
	private PurchaseNoticeService purchaseNoticeService;

	/**
	 * 移动端，作为卖家，收到的发货提醒
	 * 
	 * @param json
	 * @return
	 */
	@RequestMapping(method = RequestMethod.GET)
	@ResponseBody
	public Page<Notice> getNotices(PageParams params, String keyword, Long fromDate, Long endDate) {
		PageInfo info = new PageInfo(params);
		// 我作为卖家，把我的企业ID作为供应商ID传入
		info.expression(PredicateUtils.in("vendUU", SystemSession.getUser().getEnUUs(), false));
		return noticeService.convertNotice(purchaseNoticeService.findAllByPageInfo(info, keyword, fromDate, endDate, null));
	}

	/**
	 * 移动端，作为卖家，根据ID查找发货提醒
	 * 
	 * @param id
	 * @return
	 */
	@RequestMapping(value = "/{id}", method = RequestMethod.GET)
	@ResponseBody
	public Notice getSaleInquiryItemInById(@PathVariable("id") Long id) {
		return noticeService.convertNotice(purchaseNoticeService.findById(id));
	}

}
