package com.uas.platform.b2b.mobile.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;

import com.uas.platform.b2b.mobile.model.PurchaseChange;
import com.uas.platform.b2b.mobile.service.PurchaseChangeService;
import com.uas.platform.b2b.service.PurchaseOrderChangeService;
import com.uas.platform.b2b.support.SystemSession;
import com.uas.platform.core.model.PageInfo;
import com.uas.platform.core.model.PageParams;
import com.uas.platform.core.persistence.criteria.PredicateUtils;

/**
 * 对移动端的数据接口<br>
 * 卖家对收到的客户采购变更单的处理
 * 
 * @author yingp
 * 
 */
@Controller("mobile.SaleChangeController")
@RequestMapping("/mobile/sale/changes")
public class SaleChangeController {

	@Autowired
	private PurchaseOrderChangeService purchaseOrderChangeService;

	@Autowired
	private PurchaseChangeService purchaseChangeService;

	/**
	 * 作为卖家，收到的采购变更单
	 * 
	 * @param json
	 * @return
	 */
	@RequestMapping(method = RequestMethod.GET)
	@ResponseBody
	public Page<PurchaseChange> getReceivedPurchaseOrderChanges(PageParams params, String keyword) {
		PageInfo info = new PageInfo(params);
		// 我作为卖家，把我的企业ID作为供应商ID传入
		info.expression(PredicateUtils.in("vendUU", SystemSession.getUser().getEnUUs(), false));
		return purchaseChangeService.convertPurchaseOrderChange(purchaseOrderChangeService.findAllByPageInfo(info,
				keyword, null));
	}

	/**
	 * 作为卖家，根据变更单ID查找采购变更单(含明细)
	 * 
	 * @param id
	 * @return
	 */
	@RequestMapping(value = "/{id}", method = RequestMethod.GET)
	@ResponseBody
	public PurchaseChange getReceivedPurchaseOrderItemById(@PathVariable("id") Long id) {
		return purchaseChangeService.convertPurchaseOrderChange(purchaseOrderChangeService.findById(id));
	}

}
