package com.uas.platform.b2b.model;

import com.alibaba.fastjson.annotation.JSONField;
import org.codehaus.jackson.annotate.JsonIgnore;

import javax.persistence.*;
import java.util.*;

/**
 * 买家的委外验退单明细
 * 
 * @author aof
 * @date 2015年8月26日
 */
@Table(name = "make$returnitem")
@Entity
public class MakeReturnItem {

	/**
	 * 主键ID
	 */
	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	@Column(name = "mri_id")
	private Long id;

	/**
	 * 序号
	 */
	@Column(name = "mri_number")
	private Short number;

	/**
	 * 数量
	 */
	@Column(name = "mri_qty")
	private Double qty;

	/**
	 * 委外采购单价
	 */
	@Column(name = "mri_orderpice")
	private Double orderPrice;

	/**
	 * 税率
	 */
	@Column(name = "mri_taxrate")
	private Double taxrate;

	/**
	 * 验收单主记录
	 */
	@ManyToOne(cascade = CascadeType.ALL, optional = true)
	@JoinColumn(name = "mri_paid", nullable = false)
	private MakeReturn returns;

	/**
	 * 委外订单
	 */
	@OneToOne(cascade = { CascadeType.REFRESH })
	@JoinColumn(name = "mri_odid", insertable = false, updatable = false)
	private MakeOrder order;

	/**
	 * 委外加工单id
	 */
	@Column(name = "mri_odid")
	private Long orderId;

	/**
	 * 入库批号
	 */
	@Column(name = "mri_batchcode")
	private String batchCode;

	/**
	 * 备注
	 */
	@Column(name = "mri_remark")
	private String remark;

	/**
	 * 仓库名称
	 */
	@Column(name = "mri_whname")
	private String whname;

	/**
	 * 保存erp传入数据时间
	 */
	@Column(name = "mri_erpdate")
	private Date erpDate;

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public Short getNumber() {
		return number;
	}

	public void setNumber(Short number) {
		this.number = number;
	}

	public Double getQty() {
		return qty;
	}

	public void setQty(Double qty) {
		this.qty = qty;
	}

	public Double getOrderPrice() {
		return orderPrice;
	}

	public void setOrderPrice(Double orderPrice) {
		this.orderPrice = orderPrice;
	}

	public Double getTaxrate() {
		return taxrate;
	}

	public void setTaxrate(Double taxrate) {
		this.taxrate = taxrate;
	}

	@JsonIgnore
	@JSONField(serialize = false)
	public MakeReturn getReturn() {
		return returns;
	}

	public void setReturn(MakeReturn returns) {
		this.returns = returns;
	}

	public MakeOrder getOrder() {
		return order;
	}

	public void setOrder(MakeOrder order) {
		this.order = order;
	}

	public Long getOrderId() {
		return orderId;
	}

	public void setOrderId(Long orderId) {
		this.orderId = orderId;
	}

	public String getBatchCode() {
		return batchCode;
	}

	public void setBatchCode(String batchCode) {
		this.batchCode = batchCode;
	}

	public String getRemark() {
		return remark;
	}

	public void setRemark(String remark) {
		this.remark = remark;
	}

	public String getWhname() {
		return whname;
	}

	public void setWhname(String whname) {
		this.whname = whname;
	}

	public Date getErpDate() {
		return erpDate;
	}

	public void setErpDate(Date erpDate) {
		this.erpDate = erpDate;
	}

	/**
	 * 将明细的重复数据去除
	 * 
	 * @param items
	 * @return
	 */
	public static List<MakeReturn> distinct(List<MakeReturnItem> items) {
		List<MakeReturn> list = new ArrayList<MakeReturn>();
		Set<Long> keys = new HashSet<Long>();
		for (MakeReturnItem item : items) {
			if (!keys.contains(item.getReturn().getId())) {
				list.add(item.getReturn());
				keys.add(item.getReturn().getId());
			}
		}
		return list;
	}

}
