package com.uas.platform.b2b.model;

import com.alibaba.fastjson.annotation.JSONField;
import org.codehaus.jackson.annotate.JsonIgnore;

import javax.persistence.*;
import java.util.*;

/**
 * 买家的采购验收单明细
 * 
 * @author yingp
 * 
 */
@Table(name = "purc$acceptitem")
@Entity
public class PurchaseAcceptItem {

	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	@Column(name = "pai_id")
	private Long id;

	/**
	 * 序号
	 */
	@Column(name = "pai_number")
	private Short number;

	/**
	 * 数量
	 */
	@Column(name = "pai_qty")
	private Double qty;

	/**
	 * 采购单价
	 */
	@Column(name = "pai_orderprice")
	private Double orderPrice;

	/**
	 * 税率
	 */
	@Column(name = "pai_taxrate")
	private Double taxrate;

	/**
	 * 验收单主记录
	 */
	@ManyToOne(cascade = {CascadeType.MERGE, CascadeType.REFRESH, CascadeType.REMOVE,  CascadeType.PERSIST})
	@JoinColumn(name = "pai_paid", nullable = false)
	private PurchaseAccept accept;

	/**
	 * 采购订单明细
	 */
	@OneToOne(cascade = { CascadeType.REFRESH })
	@JoinColumn(name = "pai_pdid", insertable = false, updatable = false)
	private PurchaseOrderItem orderItem;

	@Column(name = "pai_pdid")
	private Long orderItemId;

	/**
	 * 入库批号
	 */
	@Column(name = "pai_batchcode")
	private String batchCode;

	/**
	 * 备注
	 */
	@Column(name = "pai_remark", length = 500)
	private String remark;

	/**
	 * 仓库名称
	 */
	@Column(name = "pai_whname")
	private String whname;

	/**
	 * 保存erp传入数据的时间
	 */
	@Column(name = "pai_erpdate")
	private Date erpDate;

	/**
	 * 物料资料
	 */
	@OneToOne(optional = true, cascade = { CascadeType.REFRESH })
	@JoinColumn(name = "pai_prodid", insertable = false, updatable = false, nullable = true)
	private Product product;

	@Column(name = "pai_prodid")
	private Long prodId;

	/**
	 * 物料编号
	 */
	@Column(name = "pai_prodcode")
	private String prodcode;

	/**
	 * 本次对账数
	 */
	@Column(name = "pai_ycheckqty")
	private Integer checkQty;

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public Short getNumber() {
		return number;
	}

	public void setNumber(Short number) {
		this.number = number;
	}

	public Double getQty() {
		return qty;
	}

	public void setQty(Double qty) {
		this.qty = qty;
	}

	public Double getOrderPrice() {
		return orderPrice;
	}

	public void setOrderPrice(Double orderPrice) {
		this.orderPrice = orderPrice;
	}

	@JsonIgnore
	@JSONField(serialize = false)
	public PurchaseAccept getAccept() {
		return accept;
	}

	public void setAccept(PurchaseAccept accept) {
		this.accept = accept;
	}

	public String getRemark() {
		return remark;
	}

	public void setRemark(String remark) {
		this.remark = remark;
	}

	public PurchaseOrderItem getOrderItem() {
		return orderItem;
	}

	public void setOrderItem(PurchaseOrderItem orderItem) {
		this.orderItem = orderItem;
	}

	public Long getOrderItemId() {
		return orderItemId;
	}

	public void setOrderItemId(Long orderItemId) {
		this.orderItemId = orderItemId;
	}

	public String getBatchCode() {
		return batchCode;
	}

	public void setBatchCode(String batchCode) {
		this.batchCode = batchCode;
	}

	public Double getTaxrate() {
		return taxrate;
	}

	public void setTaxrate(Double taxrate) {
		this.taxrate = taxrate;
	}

	public String getWhname() {
		return whname;
	}

	public void setWhname(String whname) {
		this.whname = whname;
	}

	public Date getErpDate() {
		return erpDate;
	}

	public void setErpDate(Date erpDate) {
		this.erpDate = erpDate;
	}

	public static List<PurchaseAccept> distinct(List<PurchaseAcceptItem> items) {
		List<PurchaseAccept> accepts = new ArrayList<PurchaseAccept>();
		Set<Long> keys = new HashSet<Long>();
		for (PurchaseAcceptItem item : items) {
			if (!keys.contains(item.getAccept().getId())) {
				accepts.add(item.getAccept());
				keys.add(item.getAccept().getId());
			}
		}
		return accepts;
	}

	public Long getProdId() {
		return prodId;
	}

	public void setProdId(Long prodId) {
		this.prodId = prodId;
	}

	public Product getProduct() {
		return product;
	}

	public void setProduct(Product product) {
		this.product = product;
	}

	public String getProdcode() {
		return prodcode;
	}

	public void setProdcode(String prodcode) {
		this.prodcode = prodcode;
	}

	public Integer getCheckQty() {
		return checkQty;
	}

	public void setCheckQty(Integer checkQty) {
		this.checkQty = checkQty;
	}
}
