package com.uas.platform.b2b.model;

import org.hibernate.annotations.Where;

import javax.persistence.*;
import java.io.Serializable;
import java.util.Date;
import java.util.Set;

/**
 * 平台里面，以供应商的角度来查看采购订单
 * 
 * @author yingp
 *
 */
@Table(name = "purc$orders", indexes = { @Index(name = "purc$orders_date", columnList = "pu_date"), @Index(name = "unique_enuu_code", columnList = "pu_enuu,pu_code", unique = true) })
@Entity
public class PurchaseOrder implements Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	@Id
	@Column(name = "pu_id")
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	private Long id;

	/**
	 * 采购单所属企业UU
	 */
	@Column(name = "pu_enuu")
	private Long enUU;

	/**
	 * 采购企业
	 */
	@OneToOne(cascade = { CascadeType.REFRESH })
	@JoinColumn(name = "pu_enuu", insertable = false, updatable = false)
	private EnterpriseInfo enterprise;

	/**
	 * 采购单所属采购员
	 */
	@OneToOne(cascade = { CascadeType.REFRESH })
	@JoinColumn(name = "pu_useruu", insertable = false, updatable = false)
	private UserBaseInfo user;

	/**
	 * 采购单所属采购员UU
	 */
	@Column(name = "pu_useruu")
	private Long userUU;

	/**
	 * 采购单号
	 */
	@Column(name = "pu_code")
	private String code;

	/**
	 * 采购类型
	 */
	@Column(name = "pu_type")
	private String type;

	/**
	 * 单据归属日期
	 */
	@Column(name = "pu_date")
	private Date date;

	/**
	 * 录单人
	 */
	@Column(name = "pu_recorder")
	private String recorder;

	/**
	 * 审核人
	 */
	@Column(name = "pu_auditor")
	private String auditor;

	/**
	 * 供应商UU
	 */
	@Column(name = "pu_venduu")
	private Long vendUU;

	/**
	 * 供应商联系人UU
	 */
	@Column(name = "pu_venduseruu")
	private Long vendUserUU;

	/**
	 * 供应商联系人<br>
	 * 平台录入时取的供应商联系人信息
	 */
	@Column(name = "pu_venduser")
	private String venduser;

	/**
	 * 收货地址
	 */
	@Column(name = "pu_shipaddress")
	private String shipAddress;

	/**
	 * 备注
	 */
	@Column(name = "pu_remark")
	private String remark;

	/**
	 * 付款币种
	 */
	@Column(name = "pu_currency")
	private String currency;

	/**
	 * 汇率
	 */
	@Column(name = "pu_rate")
	private Float rate;

	/**
	 * 付款方式
	 */
	@Column(name = "pu_payments")
	private String payments;

	/**
	 * 处理状态（已回复、未回复），全部回复后改为已回复
	 */
	@Column(name = "pu_status")
	private Short status;

	/**
	 * 从平台传到供应商ERP的状态
	 */
	@Column(name = "pu_sendstatus")
	private Short sendStatus;
	
	/**
	 * 1表示该采购单的明细至少有一条全部回复完的情况
	 */
	@Column(name = "pu_reply")
	private Short reply;
	
	/**
	 * 1表示该采购单的明细至少有一条结案的情况
	 */
	@Column(name = "pu_end")
	private Short end;
	
	/**
	 * 显示状态(0 已查看 ， 1 未查看， 2 置顶)
	 */
	@Column(name = "pu_display")
	private Short display;
	
	/**
	 * 打印次数（供应商）
	 */
	@Column(name = "pu_print")
	private Short print;

	/**
	 * 打印次数(采购方)
	 */
	@Column(name = "pu_custprint")
	private Short custPrint;

	/**
	 * 应付供应商名称
	 */
	@Column(name = "pu_receivename")
	private String receiveName;

	/**
	 * 客户公司名称
	 */
	@Column(name = "pu_cop")
	private String cop;

	/**
	 * 应付供应商编号
	 */
	@Column(name = "pu_receivecode")
	private String receiveCode;

	/**
	 * 对应erpid
	 */
	@Column(name = "pu_erpid")
	private Long erpId;

	/**
	 * 运输方式
	 */
	@Column(name = "pu_transport")
	private String transport;

	public Short getCustPrint() {
		return custPrint;
	}

	public void setCustPrint(Short custPrint) {
		this.custPrint = custPrint;
	}

	/**
	 * 打印日志
	 */
	@SuppressWarnings("deprecation")
	@OneToMany(fetch = FetchType.EAGER)
	@JoinColumn(name = "pl_foreign", insertable = false, updatable = false)
	@org.hibernate.annotations.ForeignKey(name = "none")
	@Where(clause = "pl_table = 'purc$orders'")
	private Set<PrintLog> printLogs;

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public Long getEnUU() {
		return enUU;
	}

	public void setEnUU(Long enUU) {
		this.enUU = enUU;
	}

	public Long getUserUU() {
		return userUU;
	}

	public void setUserUU(Long userUU) {
		this.userUU = userUU;
	}

	public String getCode() {
		return code;
	}

	public void setCode(String code) {
		this.code = code;
	}

	public String getType() {
		return type;
	}

	public void setType(String type) {
		this.type = type;
	}

	public Date getDate() {
		return date;
	}

	public void setDate(Date date) {
		this.date = date;
	}

	public String getRecorder() {
		return recorder;
	}

	public void setRecorder(String recorder) {
		this.recorder = recorder;
	}

	public String getAuditor() {
		return auditor;
	}

	public void setAuditor(String auditor) {
		this.auditor = auditor;
	}

	public Long getVendUU() {
		return vendUU;
	}

	public void setVendUU(Long vendUU) {
		this.vendUU = vendUU;
	}

	public Long getVendUserUU() {
		return vendUserUU;
	}

	public void setVendUserUU(Long vendUserUU) {
		this.vendUserUU = vendUserUU;
	}

	public String getShipAddress() {
		return shipAddress;
	}

	public void setShipAddress(String shipAddress) {
		this.shipAddress = shipAddress;
	}

	public String getRemark() {
		return remark;
	}

	public void setRemark(String remark) {
		this.remark = remark;
	}

	public String getCurrency() {
		return currency;
	}

	public void setCurrency(String currency) {
		this.currency = currency;
	}

	public Float getRate() {
		return rate;
	}

	public void setRate(Float rate) {
		this.rate = rate;
	}

	public String getPayments() {
		return payments;
	}

	public void setPayments(String payments) {
		this.payments = payments;
	}

	public Short getStatus() {
		return status;
	}

	public void setStatus(Short status) {
		this.status = status;
	}

	public Short getSendStatus() {
		return sendStatus;
	}

	public void setSendStatus(Short sendStatus) {
		this.sendStatus = sendStatus;
	}

	public EnterpriseInfo getEnterprise() {
		return enterprise;
	}

	public void setEnterprise(EnterpriseInfo enterprise) {
		this.enterprise = enterprise;
	}

	public UserBaseInfo getUser() {
		return user;
	}

	public void setUser(UserBaseInfo user) {
		this.user = user;
	}

	public Short getReply() {
		return reply;
	}

	public void setReply(Short reply) {
		this.reply = reply;
	}

	public Short getEnd() {
		return end;
	}

	public void setEnd(Short end) {
		this.end = end;
	}

	public Short getDisplay() {
		if(this.display == null) {
            this.display = 0;
        }
		return display;
	}

	public void setDisplay(Short display) {
		this.display = display;
	}

	public Short getPrint() {
		return print;
	}

	public void setPrint(Short print) {
		this.print = print;
	}

	public String getReceiveName() {
		return receiveName;
	}

	public void setReceiveName(String receiveName) {
		this.receiveName = receiveName;
	}

	public String getReceiveCode() {
		return receiveCode;
	}

	public void setReceiveCode(String receiveCode) {
		this.receiveCode = receiveCode;
	}

	public Set<PrintLog> getPrintLogs() {
		return printLogs;
	}

	public void setPrintLogs(Set<PrintLog> printLogs) {
		this.printLogs = printLogs;
	}

	public Long getErpId() {
		return erpId;
	}

	public void setErpId(Long erpId) {
		this.erpId = erpId;
	}

	public String getVenduser() {
		return venduser;
	}

	public void setVenduser(String venduser) {
		this.venduser = venduser;
	}

	public String getCop() {
		return cop;
	}

	public void setCop(String cop) {
		this.cop = cop;
	}

	public String getTransport() {
		return transport;
	}

	public void setTransport(String transport) {
		this.transport = transport;
	}
}
